==> emencia-crma-2.2.0.tar.gz: emencia-crma-2.2.0/crma/management/mailer.py <==
# -*- coding: utf-8 -*-

import smtplib
import logging
import traceback

from django.utils import timezone as tz
from django.conf import settings
from django.contrib.sites.models import Site

from crma.models import EmailScheduler, ST_SENT, ST_ERROR
from crma.mails import MailBuilder
from crma.signals import email_sent, emails_sent
from django.core import mail


logger = logging.getLogger(__name__)


class SendableMessage(object):
    def __init__(self, email_scheduler):
        self.email_scheduler = email_scheduler
        self.message = None

    def __repr__(self):
        return 'M<{!r}>'.format(self.email_scheduler)

    def _set_error(self, trace):
        self.email_scheduler.status = ST_ERROR
        self.email_scheduler.trace_error = traceback.format_exc()
        self.email_scheduler.save()

    def prepare(self, debug):
        try:
            mb = MailBuilder(self.email_scheduler, Site.objects.get_current())
            message = mb.build_email()
        except Exception as e:
            logger.exception('Error preparing %s', self)
            self._set_error(e)
            raise
        else:
            if debug:
                message.to = [settings.CRMA_DEBUG_EMAIL]
        self.message = message
        return message

    def send(self, connection):
        assert self.message, 'Message has not been prepared'
        try:
            connection.send_messages([self.message])
        except smtplib.SMTPRecipientsRefused as e:
            logger.error('Error sending %s', self)
            self._set_error(e)
            raise
        except smtplib.SMTPResponseException as e:
            logger.error('Error sending %s', self)
            if e.smt.p_code >= 500:
                self._set_error(e)
            raise
        except Exception as e:
            logger.exception('Error sending %s', self)
            self._set_error(e)
            raise
        else:
            self._set_sent()
        email_sent.send(sender=EmailScheduler, data={
            'id': self.email_scheduler.id,
            'email_scheduler': self.email_scheduler,
            'message': self.message,
        })

    def _set_sent(self):
        self.email_scheduler.status = ST_SENT
        self.email_scheduler.sent_time = tz.now()
        self.email_scheduler.save()


class Mailer(object):
    def __init__(self, debug=False, date_fn=tz.now):
        self.date_fn = date_fn
        self.debug = debug

    def __call__(self):
        # Emails to send
        now = self.date_fn()
        email_schedulers = EmailScheduler.objects.sendable(now)
        prepared_emails = self.prepare_emails(email_schedulers)
        if not prepared_emails:
            return

        logger.info('%s - %s prepared emails', now, len(prepared_emails))
        with mail.get_connection() as connection:
            sent_mails = self.send_messages(connection, prepared_emails)
        logger.info('%s - %s sent emails', now, len(sent_mails))

        emails_sent.send(sender=EmailScheduler,
                         prepared_emails=prepared_emails,
                         emails=sent_mails,
                         )

    def prepare_emails(self, email_schedulers):
        if email_schedulers.count() == 0:
            return []

        messages = []
        for email_scheduler in email_schedulers.iterator():
            sm = SendableMessage(email_scheduler)
            try:
                sm.prepare(self.debug)
            except Exception as e:
                logger.warning('Cannot prepare %s: %s', email_scheduler, e)
            else:
                messages.append(sm)
        return messages

    def send_messages(self, connection, messages):
        sent_messages = []
        for message in messages:
            try:
                message.send(connection)
            except Exception as e:
                logger.warning('Cannot send %s: %s', message, e)
            else:
                sent_messages.append(message)
        return sent_messages

==> emencia-crma-2.2.1.tar.gz: emencia-crma-2.2.1/crma/management/mailer.py <==
# -*- coding: utf-8 -*-

import smtplib
import logging
import traceback

from django.utils import timezone as tz
from django.conf import settings
from django.contrib.sites.models import Site

from crma.models import EmailScheduler, ST_SENT, ST_ERROR
from crma.mails import MailBuilder
from crma.signals import email_sent, emails_sent
from django.core import mail


logger = logging.getLogger(__name__)


class SendableMessage(object):
    def __init__(self, email_scheduler):
        self.email_scheduler = email_scheduler
        self.message = None

    def __repr__(self):
        return 'M<{!r}>'.format(self.email_scheduler)

    def _set_error(self, trace):
        self.email_scheduler.status = ST_ERROR
        self.email_scheduler.trace_error = traceback.format_exc()
        self.email_scheduler.save()

    def prepare(self, debug):
        try:
            mb = MailBuilder(self.email_scheduler, Site.objects.get_current())
            message = mb.build_email()
        except Exception as e:
            logger.exception('Error preparing %s', self)
            self._set_error(e)
            raise
        else:
            if debug:
                message.to = [settings.CRMA_DEBUG_EMAIL]
        self.message = message
        return message

    def send(self, connection):
        assert self.message, 'Message has not been prepared'
        try:
            connection.send_messages([self.message])
        except smtplib.SMTPServerDisconnected:
            raise
        except smtplib.SMTPRecipientsRefused as e:
            logger.error('Error sending %s', self)
            self._set_error(e)
            raise
        except smtplib.SMTPResponseException as e:
            logger.error('Error sending %s', self)
            if e.smtp_code >= 500:
                self._set_error(e)
            raise
        except Exception as e:
            logger.exception('Error sending %s', self)
            self._set_error(e)
            raise
        else:
            self._set_sent()
        email_sent.send(sender=EmailScheduler, data={
            'id': self.email_scheduler.id,
            'email_scheduler': self.email_scheduler,
            'message': self.message,
        })

    def _set_sent(self):
        self.email_scheduler.status = ST_SENT
        self.email_scheduler.sent_time = tz.now()
        self.email_scheduler.save()


class Mailer(object):
    def __init__(self, debug=False, date_fn=tz.now):
        self.date_fn = date_fn
        self.debug = debug

    def __call__(self):
        # Emails to send
        now = self.date_fn()
        email_schedulers = EmailScheduler.objects.sendable(now)
        prepared_emails = self.prepare_emails(email_schedulers)
        if not prepared_emails:
            return

        logger.info('%s - %s prepared emails', now, len(prepared_emails))
        connection = mail.get_connection()

        iterator = iter(prepared_emails)
        while True:
            try:
                connection.open()
                sent_mails = self.send_messages(connection, iterator)
            except smtplib.SMTPServerDisconnected:
                continue
            else:
                break
            finally:
                connection.close()

        logger.info('%s - %s sent emails', now, len(sent_mails))

        emails_sent.send(sender=EmailScheduler,
                         prepared_emails=prepared_emails,
                         emails=sent_mails,
                         )

    def prepare_emails(self, email_schedulers):
        if email_schedulers.count() == 0:
            return []

        messages = []
        for email_scheduler in email_schedulers.iterator():
            sm = SendableMessage(email_scheduler)
            try:
                sm.prepare(self.debug)
            except Exception as e:
                logger.warning('Cannot prepare %s: %s', email_scheduler, e)
            else:
                messages.append(sm)
        return messages

    def send_messages(self, connection, messages):
        sent_messages = []
        for message in messages:
            try:
                message.send(connection)
            except smtplib.SMTPServerDisconnected:
                raise
            except Exception as e:
                logger.warning('Cannot send %s: %s', message, e)
            else:
                sent_messages.append(message)
        return sent_messages
